@@ -13,6 +13,8 @@
 #' function allows a `brms` model to be fit with the ordered beta regression
 #' distribution.
 #'
+#' For more information about the model, see the paper here: https://osf.io/preprints/socarxiv/2sx6y/.
+#'
 #' This function allows you to set priors on the dispersion parameter,
 #' the cutpoints, and the regression coefficients (see below for options).
 #' However, to add specific priors on individual covariates, you would need
@@ -29,6 +31,10 @@
 #'   [brms::bf] function. *Please avoid using 0 or `Intercept` in the
 #'   formula definition.
 #' @param data An R data frame or tibble containing the variables in the formula
+#' @param true_bounds If the true bounds of the outcome/response
+#'   don't exist in the data, pass a length 2 numeric vector
+#'   of the minimum and maximum bounds to properly normalize
+#'   the outcome/response
 #' @param use_brm_multiple (T/F) Whether the model should use
 #'   [brms::brm_multiple] for multiple
 #'   imputation over multiple dataframes passed
@@ -45,10 +51,11 @@
 #' @param phi_prior The mean parameter of the exponential prior on
 #'  phi, which determines the dispersion of the beta distribution. The
 #'  default is .1, which equals a mean of 10 and is thus weakly
-#'  informativce. If the response has very low variance (i.e. tightly)
-#'  clusters around a specific value, then increasing this prior may be
+#'  informative on the interval (0.4, 30). If the response has very low variance (i.e. tightly)
+#'  clusters around a specific value, then decreasing this prior (and increasing the expected value)
+#'  may be
 #'  helpful. Checking the value of phi in the output of the model command
-#'  will reveal if 10 is too small.
+#'  will reveal if a value of 0.1 (mean of 10) is too small.
 #' @param dirichlet_prior A vector of three integers
 #'   corresponding to the prior parameters for the dirchlet distribution
 #'   (alpha parameter) governing the location of the cutpoints between
@@ -65,12 +72,12 @@
 #'   regression coefficients for predicting phi, the dispersion parameter.
 #'   Only useful if a linear model is being fit to phi.
 #'   Default is 5, which makes the prior weakly informative on the
-#'   logit scale.
+#'   exponential scale.
 #' @param extra_prior An additional prior, such as a prior for a specific
 #'  regression coefficient, added to the model by passing one of the `brms`
 #'  functions [brms::set_prior] or [brms::prior_string] with appropriate
 #'  values.
-#' @param inits This parameter is used to determine starting values for
+#' @param init This parameter is used to determine starting values for
 #'   the Stan sampler to begin Markov Chain Monte Carlo sampling. It is
 #'   set by default at 0 because the non-linear nature of beta regression
 #'   means that it is possible to begin with extreme values depending on the
@@ -80,6 +87,7 @@
 #'   with an experimental feature of `brms`, set this to `"random"` to get
 #'   more robust starting values (just be sure to scale the covariates so they are
 #'   not too large in absolute size).
+#' @param inits Alias of `init`. Overrides `init` when not NULL.
 #' @param ... All other arguments passed on to the `brm` function
 #' @return A `brms` object fitted with the ordered beta regression distribution.
 #' @examples
@@ -117,6 +125,7 @@
 #' @export
 ordbetareg <- function(formula=NULL,
                        data=NULL,
+                       true_bounds=NULL,
                        phi_reg=FALSE,
                        use_brm_multiple=FALSE,
                        coef_prior_mean=0,
@@ -126,9 +135,14 @@ ordbetareg <- function(formula=NULL,
                        phi_coef_prior_mean=0,
                        phi_coef_prior_sd=5,
                        extra_prior=NULL,
-                       inits="0",
+                       init ="0",
+                       inits = NULL,
                        ...) {
 
+  if(!is.null(inits)){
+    init <- inits
+  }
+
 
   if(is.null(formula)) {
 
@@ -141,7 +155,7 @@ ordbetareg <- function(formula=NULL,
 
     dv <- all.vars(formula$formula)[1]
 
-    formula$formula <- .update2.formula(formula$formula, "0 + Intercept + ")
+    formula$formula <- .update2.formula(formula$formula, "0 + Intercept ")
 
   } else if('mvbrmsformula' %in% class(formula)) {
 
@@ -153,7 +167,13 @@ ordbetareg <- function(formula=NULL,
 
     formula$forms <- lapply(formula$forms, function(var) {
 
-      var$formula <- .update2.formula(var$formula, "0 + Intercept + ")
+      if(is.null(var$family)) {
+
+        var$formula <- .update2.formula(var$formula, "0 + Intercept ")
+
+      }
+
+        var
 
     })
 
@@ -161,7 +181,7 @@ ordbetareg <- function(formula=NULL,
 
     dv <- all.vars(formula)[1]
 
-    formula <- .update2.formula(formula, "0 + Intercept + ")
+    formula <- .update2.formula(formula, "0 + Intercept ")
 
   }
 
@@ -171,11 +191,13 @@ ordbetareg <- function(formula=NULL,
 
   }
 
+  all_fam_types <- sapply(formula$forms, function(var) var$family$family)
+
   # figure out where it is so we can edit it
 
   if(use_brm_multiple) {
 
-    if(!is.list(data))
+    if(is.data.frame(data))
       stop("To use brm_multiple with ordbetareg, please pass the multiple imputed datasets as a list to the data argument.\nMice objects are not currently supported.")
 
     if(length(dv)==1) {
@@ -187,7 +209,7 @@ ordbetareg <- function(formula=NULL,
 
         if(!(all(d[[dv_pos]] >= 0 & d[[dv_pos]] <= 1,na.rm=T))) {
 
-          d[[dv_pos]] <- normalize(d[[dv_pos]])
+          d[[dv_pos]] <- normalize(d[[dv_pos]],true_bounds=true_bounds)
 
         } else {
 
@@ -212,21 +234,28 @@ ordbetareg <- function(formula=NULL,
 
       data <- lapply(data, function(d) {
 
-        d <- lapply(1:length(d), function(c) {
+        d_prime <- lapply(1:length(d), function(c) {
 
           if(c %in% dv_pos) {
 
-            if(!(all(d[[c]] >= 0 & d[[c]] <= 1,na.rm=T))) {
+            if(is.null(formula$forms[[which(dv_pos==c)]]$family)) {
 
-              out_var <- normalize(d[[c]])
+              if(!(all(d[[c]] >= 0 & d[[c]] <= 1,na.rm=T))) {
 
+                out_var <- normalize(d[[c]],true_bounds=true_bounds)
+
+              } else {
+
+                out_var <- d[[c]]
+
+                attr(out_var, "upper_bound") <- 1
+                attr(out_var, "lower_bound") <- 0
+
+              }
             } else {
 
               out_var <- d[[c]]
 
-              attr(out_var, "upper_bound") <- 1
-              attr(out_var, "lower_bound") <- 0
-
             }
 
           } else {
@@ -239,7 +268,9 @@ ordbetareg <- function(formula=NULL,
 
         })
 
-        d
+        names(d_prime) <- names(d)
+
+        d_prime
 
       })
 
@@ -260,7 +291,7 @@ ordbetareg <- function(formula=NULL,
 
       if(!(all(data[[dv_pos]] >= 0 & data[[dv_pos]] <= 1,na.rm=T))) {
 
-        data[[dv_pos]] <- normalize(data[[dv_pos]])
+        data[[dv_pos]] <- normalize(data[[dv_pos]],true_bounds=true_bounds)
 
       } else {
 
@@ -281,21 +312,28 @@ ordbetareg <- function(formula=NULL,
 
       # check all outcomes
 
-      data <- lapply(1:length(data), function(c) {
+      d_prime <- lapply(1:length(data), function(c) {
 
         if(c %in% dv_pos) {
 
-          if(!(all(data[[c]] >= 0 & data[[c]] <= 1,na.rm=T))) {
+          if(is.null(formula$forms[[which(dv_pos==c)]]$family)) {
 
-            out_var <- normalize(data[[c]])
+            if(!(all(data[[c]] >= 0 & data[[c]] <= 1,na.rm=T))) {
 
+              out_var <- normalize(data[[c]],true_bounds=true_bounds)
+
+            } else {
+
+              out_var <- data[[c]]
+
+              attr(out_var, "upper_bound") <- 1
+              attr(out_var, "lower_bound") <- 0
+
+            }
           } else {
 
             out_var <- data[[c]]
 
-            attr(out_var, "upper_bound") <- 1
-            attr(out_var, "lower_bound") <- 0
-
           }
 
         } else {
@@ -308,18 +346,50 @@ ordbetareg <- function(formula=NULL,
 
       })
 
+      names(d_prime) <- names(data)
+
+      data <- d_prime
+
     }
 
 
 
   }
 
-
-
-
   # get ordered beta regression definition
 
 
+  sep_fam <- F
+  suffix <- ""
+
+
+  if('mvbrmsformula' %in% class(formula)) {
+    # update formula objects with model families if they
+    # are distinct families
+
+    suffix <- paste0("_",names(all_fam_types))
+
+    sep_fam <- (sum(sapply(all_fam_types, function(a) !is.null(a)))>0)
+
+    if(sep_fam) {
+
+      need_resp <- formula$resp[sapply(all_fam_types, is.null)]
+
+    }
+
+  #   if(any(!sapply(all_fam_types, is.null))) {
+  #
+  #     sep_fam <- T
+  #     need_resp <- formula$resp[sapply(all_fam_types, is.null)]
+  #     suffix <-
+  #   } else {
+  #     suffix <- ""
+  #   }
+  # } else {
+  #
+   }
+
+
 
   ordbeta_mod <- .load_ordbetareg(beta_prior=c(coef_prior_mean,
                                                coef_prior_sd),
@@ -327,52 +397,42 @@ ordbetareg <- function(formula=NULL,
                                                         phi_coef_prior_sd),
                                   dirichlet_prior_num=dirichlet_prior,
                                   phi_prior = phi_prior,
-                                  extra_prior=extra_prior)
+                                  extra_prior=extra_prior,
+                                  suffix=suffix)
 
+  if('mvbrmsformula' %in% class(formula)) {
+    # update formula objects with model families if they
+    # are distinct families
 
-  if(use_brm_multiple) {
+    if(sep_fam) {
+      formula$forms <- lapply(formula$forms, function(var) {
 
-    if(phi_reg) {
+        if(!is.null(var$family)) {
 
-      brm_multiple(formula=formula, data=data,
-                   stanvars=ordbeta_mod$stanvars,
-                   family=ordbeta_mod$family,
-                   prior=ordbeta_mod$priors_phireg,
-                   inits=inits,
-                   ...)
+          return(var)
 
-    } else {
+        } else {
 
-      brm_multiple(formula=formula, data=data,
-                   stanvars=ordbeta_mod$stanvars,
-                   family=ordbeta_mod$family,
-                   prior=ordbeta_mod$priors,
-                   inits=inits,
-                   ...)
+          # add in all ordbetareg details here
 
-    }
+          var <- bf(var$formula,
+                    family=ordbeta_mod$family)
 
+        }
 
+      })
 
-  } else {
+      # remove any ordbetareg priors for diff families
 
-    if(phi_reg) {
+      all_fam_types <- ! (sapply(all_fam_types, is.null))
 
-      brm(formula=formula, data=data,
-          stanvars=ordbeta_mod$stanvars,
-          family=ordbeta_mod$family,
-          prior=ordbeta_mod$priors_phireg,
-          inits=inits,
-          ...)
+      ordbeta_mod$priors <- filter(ordbeta_mod$priors, !(grepl(x=class,
+                                                               pattern="cutzero|cutone|phi") & all_fam_types[resp]))
 
-    } else {
+      # get the outcome we need to change the priors
+
+      #ordbeta_mod$priors$resp <- c(need_resp,need_resp,"",need_resp)
 
-      brm(formula=formula, data=data,
-          stanvars=ordbeta_mod$stanvars,
-          family=ordbeta_mod$family,
-          prior=ordbeta_mod$priors,
-          inits=inits,
-          ...)
 
     }
 
@@ -382,19 +442,107 @@ ordbetareg <- function(formula=NULL,
 
 
 
+
+
+  if(use_brm_multiple) {
+
+    if(phi_reg) {
+
+      out_obj <- brm_multiple(formula=formula, data=data,
+                   stanvars=ordbeta_mod$stanvars,
+                   family=ordbeta_mod$family,
+                   prior=ordbeta_mod$priors_phireg,
+                   inits=init,
+                   ...)
+
+    } else {
+
+      if(sep_fam) {
+
+        out_obj <- brm_multiple(formula=formula, data=data,
+                     stanvars=ordbeta_mod$stanvars,
+                     prior=ordbeta_mod$priors,
+                     inits=init,
+                     ...)
+
+      } else {
+
+        out_obj <- brm_multiple(formula=formula, data=data,
+                     stanvars=ordbeta_mod$stanvars,
+                     family=ordbeta_mod$family,
+                     prior=ordbeta_mod$priors,
+                     inits=init,
+                     ...)
+
+
+      }
+
+
+
+    }
+
+
+
+  } else {
+
+    if(phi_reg) {
+
+      out_obj <- brm(formula=formula, data=data,
+          stanvars=ordbeta_mod$stanvars,
+          family=ordbeta_mod$family,
+          prior=ordbeta_mod$priors_phireg,
+          inits=init,
+          ...)
+
+    } else {
+
+      if(sep_fam) {
+
+        out_obj <- brm(formula=formula, data=data,
+            stanvars=ordbeta_mod$stanvars,
+            prior=ordbeta_mod$priors,
+            inits=init,
+            ...)
+
+
+      } else {
+
+        out_obj <- brm(formula=formula, data=data,
+            stanvars=ordbeta_mod$stanvars,
+            family=ordbeta_mod$family,
+            prior=ordbeta_mod$priors,
+            inits=init,
+            ...)
+
+
+      }
+
+
+
+    }
+
+  }
+
+
+
+    class(out_obj) <- c(class(out_obj),"ordbetareg")
+
+    return(out_obj)
+
 }
 
 
 #' Internal Function to Add Ordered Beta Regression Family
 #'
 #' Not exported.
-#' @importFrom brms custom_family stanvar set_prior
+#' @importFrom brms custom_family stanvar set_prior posterior_predict
 #' @noRd
 .load_ordbetareg <- function(beta_prior=NULL,
                              phireg_beta_prior=NULL,
                              dirichlet_prior_num=NULL,
                              phi_prior=NULL,
-                             extra_prior=NULL) {
+                             extra_prior=NULL,
+                             suffix="") {
 
   # function called primarily for its side effects
 
@@ -431,6 +579,19 @@ ordbetareg <- function(formula=NULL,
   # For pulling posterior predictions
 
   posterior_predict_ord_beta_reg <- function(i, draws, ...) {
+
+    get_args <- list(...)
+
+    if(!is.null(get_args$ntrys) && get_args$ntrys>5) {
+
+      type <- "continuous"
+
+    } else {
+
+      type <- "combined"
+
+    }
+
     mu <- draws$dpars$mu[, i]
     phi <- draws$dpars$phi
     cutzero <- draws$dpars$cutzero
@@ -450,15 +611,23 @@ ordbetareg <- function(formula=NULL,
       sample(1:3,size=1,prob=c(pr_y0[i],pr_cont[i],pr_y1[i]))
     })
 
-    final_out <- sapply(1:length(outcomes),function(i) {
-      if(outcomes[i]==1) {
-        return(0)
-      } else if(outcomes[i]==2) {
-        return(out_beta[i])
-      } else {
-        return(1)
-      }
-    })
+    if(type=="combined") {
+
+        final_out <- sapply(1:length(outcomes),function(i) {
+          if(outcomes[i]==1) {
+            return(0)
+          } else if(outcomes[i]==2) {
+            return(out_beta[i])
+          } else {
+            return(1)
+          }
+        })
+
+      } else if (type=="continuous") {
+
+        final_out <- out_beta
+
+    }
 
     final_out
 
@@ -597,14 +766,55 @@ ordbetareg <- function(formula=NULL,
   # which represent regression coefficients on the logit
   # scale
 
-  priors <- set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
-                                                          collapse=","),"]', 0, 1, cutzero, cutone)"),
-                      class="cutzero") +
-    set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
-                                                  collapse=","),"]', 0, 2, cutzero, cutone)"),
-              class="cutone") +
-    set_prior(paste0("normal(",beta_prior[1],",",beta_prior[2],")"),class="b") +
-    set_prior(paste0("exponential(",phi_prior,")"),class="phi")
+  if(length(suffix)>1) {
+
+    # multiple outcomes
+
+    cutzero <- paste0("cutzero",suffix)
+    cutone <- paste0("cutone",suffix)
+
+    priors <- set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
+                                                            collapse=","),"]', 0, 1,", cutzero[1],",", cutone[1],")"),
+                        class="cutzero",resp=substring(suffix[1],2)) +
+      set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
+                                                    collapse=","),"]', 0, 2,", cutzero[1],",", cutone[1],")"),
+                class="cutone",resp=substring(suffix[1],2)) +
+      set_prior(paste0("normal(",beta_prior[1],",",beta_prior[2],")"),class="b",resp=substring(suffix[1],2)) +
+      set_prior(paste0("exponential(",phi_prior,")"),class="phi",resp=substring(suffix[1],2)) +
+      set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
+                                                    collapse=","),"]', 0, 1,", cutzero[2],",", cutone[2],")"),
+                class="cutzero",resp=substring(suffix[2],2)) +
+      set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
+                                                    collapse=","),"]', 0, 2,", cutzero[2],",", cutone[2],")"),
+                class="cutone",resp=substring(suffix[2],2)) +
+      set_prior(paste0("normal(",beta_prior[1],",",beta_prior[2],")"),class="b",resp=substring(suffix[2],2)) +
+      set_prior(paste0("exponential(",phi_prior,")"),class="phi",resp=substring(suffix[2],2))
+
+    priors_phireg <- set_prior("normal(0,5)",class="b") +
+      set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
+                                                    collapse=","),"]', 0, 1, cutzero, cutone)"),
+                class="cutzero") +
+      set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
+                                                    collapse=","),"]', 0, 2, cutzero, cutone)"),
+                class="cutone")
+
+  } else {
+
+
+    cutzero <- paste0("cutzero",suffix)
+    cutone <- paste0("cutone",suffix)
+
+    priors <- set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
+                                                            collapse=","),"]', 0, 1,", cutzero,",", cutone,")"),
+                        class="cutzero") +
+      set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
+                                                    collapse=","),"]', 0, 2,", cutzero,",", cutone,")"),
+                class="cutone") +
+      set_prior(paste0("normal(",beta_prior[1],",",beta_prior[2],")"),class="b") +
+      set_prior(paste0("exponential(",phi_prior,")"),class="phi")
+
+
+  }
 
   priors_phireg <- set_prior("normal(0,5)",class="b") +
     set_prior(paste0("induced_dirichlet([",paste0(dirichlet_prior_num,
@@ -614,9 +824,12 @@ ordbetareg <- function(formula=NULL,
                                                   collapse=","),"]', 0, 2, cutzero, cutone)"),
               class="cutone")
 
+
+
   if(!is.null(extra_prior)) {
 
     priors <- priors + extra_prior
+    priors_phireg <- priors_phireg + extra_prior
 
   }
 
